pro andril_SXT,	nfil,x0,y0,cimage,dimage,decon,etime,timek,stimek,datek,framek,$
	MODE=mode,NITER=niter,N=n,T=t,B=b,DIMPSF=dimpsf,DIV=div,THR=thr,SILENT=silent,DN=DN
;+
;NAME
;	ANDRIL_SXT
;	Version 3.1 11 May, 1999
;
;PURPOSE
;	SXT interface to ANDRIL which directly corresponds to those described in the
;	Main routine; sets-up deconvolution of  SXT images
;	from full-resolution data.
;
;CALLING SEQUENCE:
;	andril_SXT, nfil, x0 , y0 ,  cimage, dimage, decon
;	andril_SXT, nfil, x0 , y0 ,  cimage, dimage, decon,MODE=mode,NITER=niter,N=n,T=t,B=b,DIMPSF=dimpsf,DIV=div,THR=thr,/SILENT=silent
;
;	andril_sxt, 4   , 452, 496,  data_be(*,*,0), dark_be(*,*,0),decon
; 	andril_sxt, 4   , 452, 496,  data_be(*,*,0), dark_be(*,*,0),decon,niter=10,/SILENT
;
;INPUTS:
;
;	nfil	-	is the filter number for the image according to convention
;                   1:  OPTICAL'
;                   2:  THIN ALUMINUM Al 01
;                   3:  AlMg dagwood
;                   4:  Be 119
;                   5:  Al 12
;                   6:  Mg 03
;
;	x0,y0	 -	are the coordinates of the lower left corner of the SXT image to be deconvolved
;				(as obtained from gt_corner(index) fi. 816,448 for region close to the western limb)
;	cimage	 -	compressed SXT image as coming from .yodat spr* file reformatting MUST be square array
;	dimage	 -	corresponding dark current & other corrections combined array
;	etime	 -	exposure time in s used to normalize the image to rates/subpixel
;	timek	 -	time of the frame "k" in decimak hours
;	stimek	 -	string of time for frame "k" in hh:mm:ss
;	datek	 -	string of date for frame "k" in dd-mmm-yy
;	framek	 -	frame # "k"
;
;OUTPUTS:
;	decon	 - 	deconvolved image
;
;KEYWORDS:
;	MODE    -   is the version: 0 for classical Richardson-Lucy (RL),
;					1 for accelerated RL
;	NITER   -   is the termination step for the iteration
;	N,T     -   are the parameters regulating the U - smoothing  (cf. N=3, T=0.2)
;	B    -   is the pedestal background ( b =~10 ) which damps the noise
;	DIMPSF  -	is the size of the PSF array as expressed in SXT 7, for 7x7 PSF  window size
;	DIV    -	is the superresolution factor (usually 5)
;	THR    -   Signal/Noise threshold used for detecting significant pixels
;			        used for calculations of normalized Chi^2 (usulally 5)
;	SILENT  -   doesn't show the windows  for the plots: U-Function, Correction matrix, abs(obs-cal)/unc,
;		    		ratio observed/calculated and deconvolved image
;
;METHOD:
;
;PAPERS:
;	ANDRIL - Maximum Likelihood Algorithm for Deconvolution of SXT Images
;	J. S y l w e s t e r   and   B. S y l w e s t e r
;	ACTA ASTRONOMICA vol. 48 (1998)pp. 519-545
;		and
;	Reconstruction of Images with Poisson Noise
;	J. S y l w e s t e r   and   B. S y l w e s t e r
;	ACTA ASTRONOMICA (1999) - submitted
;
;
;HISTORY:
;	written J. Sylwester, 1996 - May 1999
;-

info='This programme is "REFERENCE_ware". If you present results,'
info1='please mention two references indicated at the program header'

;    		close,/all

;			checking for the mode value present
			if not keyword_set(mode) then begin
				mode=1
			endif else begin
				mode=0
			endelse

;			checking for the niter value present
			if not keyword_set(niter) then begin
				niter=50
			endif else begin
				niter=niter
			endelse

;			checking for the n value present
			if not keyword_set(n) then begin
				n=3
			endif else begin
				n=n
			endelse

;			checking for the t value present
			if not keyword_set(t) then begin
				t=0.2
			endif else begin
				t=t
			endelse

;           checking for the b value present
			if (not keyword_set(b)) and (nfil ne 1) then  b=10
			if (not keyword_set(b)) and (nfil eq 1) then  b=0
			if  keyword_set(b) then  b=b

;			checking for the dimpsf value present
			if not keyword_set(dimpsf) then begin
				dimpsf=7
			endif else begin
				dimpsf=dimpsf
			endelse

;			checking for the div value present
			if not keyword_set(div) then begin
				div=5
			endif else begin
				div=div
			endelse
;			checking for the threshold value present
			if not keyword_set(thr) then begin
				thr=5
			endif else begin
				thr=thr
			endelse
;			checking for silent condition
			if not keyword_set(silent) then begin
				print,systime(0)
				print,info
				print,info1
				tstart=systime(1)
			endif
;	Checking the errors on input images
		;checking that dark current is below reasonable limit (less 500 DN)
			warn='Problems with dark current - check the dark file'
			if max(dimage) gt 500 then stop,warn

;	Estimating the size of data array dime
			dimei=sqrt(n_elements(cimage))	; input array dimension


;	Uncompressing the data to DN's
;	special treatment of 12 bit optical data

  if (nfil eq 1) and (max(cimage) gt 255) then begin
   		obsd= cimage    ;  takes just the full accuracy
  endif else begin
     	obsd= sxt_deco(byte(cimage)) ; decompressing the signal using SXT decompression
  endelse
	;

	if (nfil eq 1) then niter=10	;forcing smaller limit no of iterations for optical filter
;	bound=(total(obsd(0,*))+ total(obsd(dimei-1,*))+$
;					total(obsd(*,0)) + total(obsd(*,dimei-1)))$
;					/(4*n_elements(obsd(0,*)))
	bound=[reform(obsd(0,*)),reform(obsd(dimei-1,*)),reform(obsd(*,0)),reform(obsd(*,dimei-1))]
;	stop
;	checking whether apply edge expansion
	if (max(bound)-min(bound) gt 10) then begin
			edge=1		;forcing edge expansion



;stop

;	Artificially Expanding the data and dark arrays
;	calculating the sizes of expanded array
			sd=dimpsf/2+1
			imag=intarr(dimei+dimpsf+1,dimei+dimpsf+1)+13	;sets up the data + boundary, fills with 13
			darkr=imag									;same for dark current
;	fills the arrays with real data inside
			imag(sd:sd+dimei-1,sd:sd+dimei-1)=obsd
			darkr(sd:sd+dimei-1,sd:sd+dimei-1)=dimage

;	calculating the PSF for pixels (psp)
			psp=sxt_psd(nfil,x0+dimei/2,y0+dimei/2,dim=dimpsf,div=1)  ; div=1 for pixels

			imag=convolve(imag,psp)		;	smoothing the edge with actual psf
			imag(sd:sd+dimei-1,sd:sd+dimei-1)=obsd	; filling inside with real data
		endif else begin
			edge=0
			imag=obsd
			darkr=dimage
		endelse
;	calculating total image DN's above dark level
;	this value will be used to setup xero approximation
	    	tc=double(total(imag)-total(darkr))
;	Re-Estimating the size of data array dime
			dime=fix(sqrt(n_elements(imag)))	; dimension to be processed including boundaries



;	UNCERTAINTY SECTION

;	setup vector containing the DN/Nphot average for 4 -20 MK range
;	see Fig. 14 in Paper I. DNN's are from Fig. 13, Paper I
    		dnn=[1,1.,3.02435,3.14247,5.59317,3.54293,2.99833]	;cf. Fig. 13 of ANDRIL
    					; this is the vector of DN/Nphotons
    					; first entry is dummy (for filter #0)
    					; second entry (1.) is for the optical filter (PSF for this filter is from sxt_psf)
    					;	similar to that for filter #2 if not identical
			        	; 1/dnn gives Nphot/DN which gives factor to convert
				        ; DN into photons - that allows for propper calculation
				        ; of uncertainties unc(Poisson) = sqrt(1/dnn*DN)=sqrt(1/dnn)*sqrt(DN)
				        ; first two entries stands for zero and optical filter
;	Restoring the DDN relative DN uncertainty file corresponds to Fig. 14 in ANDRIL
		ddnn=0
		namedd=strcompress('ddn'+string(format='(i1)',nfil)+'.sav',/remove_all) ; creating the file name
		restore,namedd			;restoring appropriate uncertainty file
		ddn=reform(ddnn(*,1))	;reform to vector

;	Special case for 12 bit optical data
		if (nfil eq 1) and (max(imag) gt 255) then begin
		   		uncer=imag/sqrt(imag/dnn(nfil))		; takes classical for 12 bit images
		endif else begin
				uncer=ddn(sxt_comp(imag))*imag	;using prescribed uncertainty function
		endelse


;	Converting DN's to photons assuming average factors
		obs=imag/dnn(nfil)      	; obs is photons/pixel

	   	dark=darkr/dnn(nfil)		; dark current in photons/pixel

		unc=uncer/dnn(nfil)			; unc is in photons/pixel


;		Setting up the flat zero approximation distribution (phi0)
		ave=tc/dnn(nfil)/(double(dime*div))^2  	; average level (photons) for
												;	zero flatt approximation
												;	(double(dime*div))^2 is the # subpixels in
												;	deconvolved image
		; checking for positivity of the X-ray signal
		if ave le 0. then begin
    		print,'Total dark current above total signal'
    		ave=positivi(ave,epsilon=1.)		; makes sure that the signal is positive
    	endif
		decon=dblarr(dime*div,dime*div)+ave 	; sets up zero approximation array
;*********************
;		Selecting pixels with Signal/Noise above 5 , i. e. 20% uncertainty
;		selects pixels above threshold WTHR IS THE SELECTION VECTOR
 		WThr=where(obs/unc ge thr)  	;select points with the accuracy better than 20%

;		Calculates PSF shape for the centre of the investigated area
;		assumes PSF shape same for all elements
;		PSF is calculated from modified LaBonte routine sxt_psd

	    	psf=sxt_psd(nfil,x0+dimei/2,y0+dimei/2,dim=dimpsf,div=div)
;		Normalizing the PSF to unity in the area of interest
			psf= double(psf)/total(psf)

;	stop


	;	Calling the R_L  ; decon is the output deconvolved image

			RL_DECOB,mode,niter,n,t,thr,div,obs,unc,dark,psf,b,decon,edge,SILENT=silent

	;	Checking whether write the results in rates DN's/s
			if keyword_set(DN) then begin

	;	rewriting into the DN's output file
	;	restoring the andril.sav file containing results in photons
			restore,'andril.sav'

				newnorm=new/etime*dnn(nfil) 	; deconvolved rates/subpixel
				darkr=dark/etime*dnn(nfil)
				imag=obsd/etime					; obsd already in DN's
				calc=calc/etime*dnn(nfil)
				uncer=uncer/etime*dnn(nfil)

		;	rewriting new info for sequence
		; 	preparing to save the results
			bb=bb/etime*dnn(nfil)		;convert pedestal level in DN's/s
			calc=calc*dnn(nfil)	;convert calculated flux in photons to DN's

		;	Preparing HEADER info
			info=strarr(38)	; 	sets up the info vector

			info(0)=' These are results of ANDRIL code developed by J. & B. Sylwester                             '
			info(1)=' If publishing the results, please mention two references:                                   '
			info(3)=' ANDRIL - Maximum Likelihood Algorithm for Deconvolution of SXT Images                       '
			info(4)=' J. S y l w e s t e r   and   B. S y l w e s t e r   (Paper I)                               '
			info(5)=' ACTA ASTRONOMICA                                                                            '
			info(6)=' Vol. 48, (1998), pp. 519-545                                                                '
			info(7)=' and                                                                                         '
			info(8)=' Reconstruction of Images with Poisson Noise                                                 '
			info(9)=' J. S y l w e s t e r   and   B. S y l w e s t e r    (Paper II)                             '
			info(10)=' ACTA ASTRONOMICA                                                                           '
			info(11)=' (1999) - submitted                                                                         '
			info(12)=' VARIABLE DESCRIPTION in this save set converted to rates [DNs/s]                                                                                           '
			info(13)=' IMAGEs ARE NORMALIZED IN DN/S ie RATES per (sub)pixel                                                                                            '
			info(14)='                               '
			info(15)=' EDGE				 0 if no edges added in calculations, 1 if extra edge                                                                                         '
			info(17)=' BB                value of "background pedestal" in DNs                            '
			info(18)=' CHITH             vector of  normalize chi^2 convergence (pixels with important signal S/N>5), if no point above threshold then the value is taken for all pixels'
			info(19)=' DARKR             array of the dark current used in DNs/s                                         '
			info(20)=' DIMPSF            size of the PSF square array used in deconvolution (input pixels)        '
			info(21)=' DIV               over-resolution factor (usually 5)                                       '
			info(22)=' IMAG              input image for deconvolution in DNs/s                          '
			info(23)=' ITER              iteration limit (or iteration step where convergence stops)              '
			info(24)=' LIKEL             vector of log L(ikelihood)  convergence                                  '
			info(25)=' N    	     	 scalar defining the smoothing properties (see Paper I)                   '
			info(26)=" NEWNORM           THE RESULT of DECONVOLUTION (DN's/s, total counts preserved)             "
			info(27)=' PSF               the PSF array used for deconvolution                                     '
			info(28)=' T                 scalar defining the smoothing properties (see Paper I)                   '
			info(29)=' THR               threshold level for S/N ratio (5 - is for 20% accuracy)                  '
			info(30)=' mode              mode used in the calculations (0 - classical RL, 1 - accelerated)        '
			info(31)=' calc              matrix of calculated fluxes DNs                               '
			info(32)=' uncer             matrix of combined total uncertainties DNs/s                                     '
			info(33)=' datek, framek     date   & frame No in the flare sequence                                                                  '
			info(34)=' timek             time, decimal hours                                                      '
			info(35)=' stimek            string with time UT                                                      '
			info(36)=' nfil              filter used                                  					          '
			info(37)=' X0,Y0             coordinates of lower left corner of the SXT FOV                                  '
	;	Storing the results to save file 'andrilm.sav'
			save,filename='andrilm.sav',bb,info,newnorm,iter,chith,imag,calc,psf,$
					mode,datek,stimek,timek,framek,nfil,likel,uncer,$
					etime,n,t,darkr,thr,x0,y0,dimpsf,div,edge

			endif  ; end writing to DN's file







		if not keyword_set(silent) then begin
			print,systime(0)
			tstop=systime(1)
			print,'calculation time = ',(tstop-tstart)/60., ' min '
		endif


	end