pro diwalim1,psf,dimpsf,div,w
;+
;NAME:
;	diwalim1
;
;PURPOSE:
;	Calculates the weight factor array by performing the
;	summation of the psf function
;	case with variable dimensions of PSF array,
;	assumes that PSF is the square array.
;
;CALLING SEQUENCE:
;	diwalim1,psf,dimpsf,div,w
;
;INPUTS:
;	psf - array of psf
;	dimpsf(Odd) - is the dimension of psf in SXT PIXELS!!
;	div - is the superresolution factor (usually 5)
;
;OUTPUTS:
;	w - four dimensional array of weights (for each
;		subpixel within the pixel element gives the
;		array of contributions to pixels within the
; 		PSF active area
;KEYWORDS:
;
;METHOD:
;	
;HISTORY:
;	written, J. Sylwester, Feb 1996
;-



;	transpose the psf array in order to insert the weights
;	in the appropriate way
	psr=rotate(psf,2)

;	setting tables for lower & upper limits

		iend= dimpsf+1  	;upper indicator for matrix dimension
							;div is for subresolution matrix
							;dimpsf/2 extends for the PSF width
							;in SXT pixels
;	calculates limits of summation:
;	gd - lower limit (granica dolna)
;	gg - upper limit (granica gorna)
		gd=intarr(div,iend+1)
		gg=gd

	for wx=0, iend do begin
		for i=0,div-1 do begin
		if wx eq 0 then gd(i,wx)=0 else gd(i,wx)=(wx-1)*div+div/2-i>0<(dimpsf*div-1)
		if wx eq iend then gg(i,wx)=(dimpsf*div-1) else gg(i,wx)=wx*div+div/2-1-i>0<(dimpsf*div-1)
		endfor
	endfor
;
;	setting array for weights ww
	w=dblarr(div,div,iend+1,iend+1)

;	considering the boundary areas
	for wx=0, iend  do begin
	for wy=0, iend  do begin
		for i=0,div-1 do begin
		for j=0,div-1 do begin
		if (wx eq 0) and (i ge div/2) or $
		   (wy eq 0) and (j ge div/2) or $
		   (wx eq iend) and (i le div/2) or $
		   (wy eq iend) and (j le div/2) then begin
		w(i,j,wx,wy)=0.
		goto, skok
		endif

		w(i,j,wx,wy)=total(psr(gd(i,wx):gg(i,wx),gd(j,wy):gg(j,wy)))
		skok:
		endfor
		endfor
	endfor
	endfor
;	additional normalization of the resulting arrays
	for i=0,div-1 do begin
	for j=0,div-1 do begin

		w(i,j,*,*)=1.d0*w(i,j,*,*)/total(w(i,j,*,*))
	endfor
	endfor
end
