function nlikely,obs,calc
;+
;NAME:
;	nlikely
;
;PURPOSE:
;	Calculates the value of 'normalized' natural LOG of the
;	likelihood function for given arrays of the 
;	observed and calculated data. It is IMPORTANT that
;	obs and calc are in photons (or statistical events) in
;	order that the likelihood be meaningfull
;
;CALLING SEQUENCE:
;	like = nlikely(obs, calc)
;
;INPUTS:
;	obs = the observed array (events)
;	calc = the calculated array
;
;OUTPUTS:
;
;KEYWORDS:
;
;METHOD:
;	Uses  Stirling approximation for
;	calculating the factorials.
;	http://ece-www.colorado.edu/~bart/book/sterling.htm
;HISTORY:
;	version 2.01 on 30 March 99,J. Sylwester, written
;-
	

	on_error,2                      ;Return to caller if an error occurs

;	check for positivity and range
	if min(obs) le 1 or max(obs) gt 1.e37 then begin
		print,'Check range of arguments'
		stop
	endif
	like = 0.5*alog(2*!dpi*obs)+obs*alog(obs)-obs + 1.d0/(12*obs)
	like = total(obs*alog(calc)-calc-like) 	; calculates total likeli
	like = like/n_elements(obs)		; per element

return,like
end


