pro RL_DECOB,mode,niter,n,t,thr,div,image,uncer,dark,psf,b,decon,edge,SILENT=silent
;+
;NAME:
;	rl_decob
;
;PURPOSE:
;	Program checks if input data for deconvolution are correct
;	is matrices are square and odd/even where nesessary
;	executes DRLB for deconvolution
;	version with pedestal b.
;
;CALLING SEQUENCE:
;	RL_DECOB,mode,niter,n,t,div,image,uncer,dark,psf,b,decon
;	RL_DECOB,mode,niter,n,t,div,image,uncer,dark,psf,b,decon,/SILENT
;
;INPUTS:
;	mode = version of deconvolution
;	niter =	final iteration step
;	n,t = are the parameters regulating the U - smoothing  (3, 0.2)
;	div = subdivision of pixels (cf, 5)
;	image = image to deconvolve (photons)
;	uncer =	uncertainty of the image (photons)
;	dark is the dark cuttent array (photons)
;	psf = point spread function shoud be given for all subdivision points
;	b = background pedestal level
;
;OUTPUTS:
;	decon 		- deconvolved image
;	andril.sav	- save file containing the results
;			  see description of variables in the info
;
;KEYWORDS:
;	SILENT   - 	does not show the diagnostic windows for the plots:
;				U-Function,
;				Correction Matrix (ww)
;				Present fit quality [ abs(obs-cal)/unc]
;				Ratio observed/calculated vs observed expressed
;					in terms of the uncertainty
;				Improved deconvolved image
;
;
;METHOD:
;
;HISTORY:
;-	J. Sylwester, May 1999, written
;


;
;	Checks for dimensions
	im=size(image)
	if im(0) ne 2 then begin
		print,'image should be array"
		goto,koniec
	endif

	if im(1) ne im(2) then begin
		print,'image should be SQUARE"
		goto,koniec
	endif

	dime=im(1)	; fixes dimension of input array
	if (dime)/2*2 ne dime then begin
		print,'image should have EVEN dimension cf 64, 66,  etc."
		goto,koniec
	endif
	iu=size(uncer)
	if iu(1) ne im(1) and iu(2) ne im(2) then begin
		print,' Image and uncertainty arrays of unequal size'
		goto,koniec
	endif
	id=size(dark)
	if iu(1) ne im(1) and iu(2) ne im(2) ne 0 then begin
		print,' Image and dark arrays  of unequal size'
		goto,koniec
	endif

	;checks psf
	ip=size(psf)
		if ip(0) ne 2 then begin
		print,'PSF should be array"
		goto,koniec
	endif

	if ip(1) ne ip(2) then begin
		print,'PSF should be SQUARE array"
		goto,koniec
	endif

	dimpsf=ip(1)	; fixes dimension of input array
	;	dimpsf should be the total size of PSF array in
	;	original pixel dimension
	dimpsf=dimpsf/div   ;	express dimpsf in original pixel size
	if (dimpsf+1)/2*2 ne dimpsf+1 then begin
		print,'PSF should have ODD dimension cf 7, 13 etc.'
		stop
		goto,koniec
	endif


	;	calling deconvolution
	DRLB,niter,mode,thr,image,uncer,dark,dime, div,n,t,psf,dimpsf,b,decon,edge,SILENT=silent

	koniec:

	end